
#include "main.h"

#include "lexer.h"
#include "parser.hpp"

#include <cassert>
#include <cstdio>
#include <iostream>

extern int yydebug;
Token token;
json::Value root;

static int test(const char *str) {
  auto buffer = yy_scan_string(str);

  auto ret = yyparse();

  yy_delete_buffer(buffer);
  printf("'%s' ret %d\n", str, ret);
  return ret;
}

#define PRINT(X) json::print(X);

int main(int argc, char **argv) {
  yydebug = 0;

  /* simpletons */
  if (auto ret = test("null"); ret != 0) {
    return ret;
  } else {
    PRINT(root);
  }

  if (auto ret = test("true"); ret != 0) {
    return ret;
  } else {
    PRINT(root);
  }

  if (auto ret = test("false"); ret != 0) {
    return ret;
  } else {
    PRINT(root);
  }

  // numbers
  if (auto ret = test("1"); ret != 0) {
    return ret;
  } else if (std::get<double>(token) != 1) {
    return 2;
  } else {
    PRINT(root);
  }

  if (auto ret = test("1."); ret != 1) {
    return 1;
  } else {
    PRINT(root);
  }

  if (auto ret = test("1.2"); ret != 0) {
    return ret;
  } else if (std::get<double>(token) != 1.2) {
    return 2;
  } else {
    PRINT(root);
  }

  if (auto ret = test(".2"); ret != 0) {
    return ret;
  } else if (std::get<double>(token) != 0.2) {
    return 2;
  } else {
    PRINT(root);
  }

  // strings
  if (auto ret = test(R"("abc")"); ret != 0) {
    return ret;
  } else if (std::get<std::string>(token) != "abc") {
    return 2;
  } else {
    PRINT(root);
  }

  if (auto ret = test(R"("abc\ncde")"); ret != 0) {
    return ret;
  } else if (std::get<std::string>(token) != "abc\ncde") {
    std::cout << "Unexpected value '" << std::get<std::string>(token) << "'"
              << std::endl;
    return 2;
  } else {
    PRINT(root);
  }

  if (auto ret = test(R"("abc\"cde")"); ret != 0) {
    return ret;
  } else if (std::get<std::string>(token) != "abc\"cde") {
    std::cout << "Unexpected value '" << std::get<std::string>(token) << "'"
              << std::endl;
    return 2;
  } else {
    PRINT(root);
  }

  // arrays
  if (auto ret = test(R"([])"); ret != 0) {
    return ret;
  } else {
    PRINT(root);
  }

  if (auto ret = test(R"([1])"); ret != 0) {
    return ret;
  } else {
    PRINT(root);
  }

  return 0;
}
