
#pragma once

#include <iostream>
#include <string>
#include <variant>
#include <vector>

using Token = std::variant<double, std::string>;
extern Token token;

namespace json {

struct Null;
struct Boolean;
struct Number;
struct String;
struct Array;
using Value = std::variant<json::Null, json::Boolean, json::Number,
                           json::String, json::Array>;

struct Null {};
struct Boolean {
  bool value;
  explicit Boolean(bool v) : value(v) {}
};

struct Number {
  double value;
  explicit Number(double v) : value(v) {}
};

struct String {
  std::string value;
  explicit String(const std::string &v) : value(v) {}
};

struct Array {
  std::vector<Value> value;
  explicit Array(const std::vector<Value> &v) : value(v) {}
  static auto is_primitive(const Value &v) {
    return std::get_if<Null>(&v) != nullptr ||
           std::get_if<Boolean>(&v) != nullptr ||
           std::get_if<Number>(&v) != nullptr ||
           std::get_if<String>(&v) != nullptr;
  }
  static Value adopt(Value &p, const Value &c);
};

inline Value Array::adopt(Value &p, const Value &c) {
  if (is_primitive(p)) {
    return Array({c});
  } else {
    std::get<Array>(p).value.push_back(c);
    return p;
  }
}

inline void print(const Value &root) {
  if (auto val = std::get_if<Null>(&root)) {
    std::cout << "Null()";
  }

  if (auto val = std::get_if<Boolean>(&root)) {
    std::cout << "Boolean(" << val->value << ")";
  }

  if (auto val = std::get_if<Number>(&root)) {
    std::cout << "Number(" << val->value << ")";
  }

  if (auto val = std::get_if<String>(&root)) {
    std::cout << "String(" << val->value << ")";
  }

  std::cout << std::endl;
}

} // namespace json

extern json::Value root;
