%{
#include "main.h"
#include "lexer.h"
void yyerror(const char *msg);
#define YYSTYPE json::Value
%}

%token T_NULL T_TRUE T_FALSE T_NUMBER T_STRING
%token T_LBRACKET T_RBRACKET T_COMMA

%%

value
    : number
    | boolean
    | null
    | string
    | array
    ;

array
    : T_LBRACKET T_RBRACKET
    | T_LBRACKET arr_values T_RBRACKET
    ;

arr_values
    : value { root = $$ = json::Array::adopt($$, $1); }
    | value T_COMMA arr_values
    ;

string
    : T_STRING { root = $$ = json::String(std::get<std::string>(token)); }
    ;

number
    :T_NUMBER { root = $$ = json::Number(std::get<double>(token)); }
    ;

boolean
    : T_TRUE  { root = $$ = json::Boolean(true); }
    | T_FALSE { root = $$ = json::Boolean(false); }
    ;


null
    : T_NULL { root = $$ = json::Null(); }
    ;

%%

void yyerror(const char *msg) {
    printf("** Error: %s\n", msg);
}
